searchData={"items":[{"type":"module","title":"OpenPGP","doc":"OpenPGP lib allows to inspect, decode and decrypt OpenPGP Message\nFormat as per [RFC4880](https://www.ietf.org/rfc/rfc4880.txt)\n\nAs of v0.5.x:\n\n  1. Any valid OpenPGP message can be decoded via generic\n     `OpenPGP.Packet` decoder. This abstraction layer provide Packet\n     Tags and Body Chunks for packet envelope level evaluation.\n  1. Some Packet Tag specific decoders implemented with limited\n     feature support:\n      1. `OpenPGP.LiteralDataPacket`\n      1. `OpenPGP.PublicKeyEncryptedSessionKeyPacket`\n      1. `OpenPGP.PublicKeyPacket` - support only V4 packets\n      1. `OpenPGP.SecretKeyPacket` - support only V4 packets; Iterated\n          and Salted String-to-Key (S2K) specifier (ID: 3); S2K usage\n          convention octet of 254 only; S2K hashing algo SHA1; AES128\n          symmetric encryption of secret key material\n      1. `OpenPGP.CompressedDataPacket` - support only ZLIB- and ZIP-\n          style blocks\n      1. `OpenPGP.IntegrityProtectedDataPacket` - support Session Key\n          algo 9 (AES with 256-bit key) in CFB mode; Modification\n          Detection Code system is not supported\n\nAt a high level `OpenPGP.list_packets/1` and `OpenPGP.cast_packets/1`\nserve as an entrypoint to OpenPGP Message decoding and extracting\ngeneric data. Packet specific decoders implement\n`OpenPGP.Packet.Behaviour`, which exposes `.decode/1` interface\n(including genric `OpenPGP.Packet`). Additionaly some of the packet\nspecific decoders may provide interface for further packet processing,\nsuch as `OpenPGP.SecretKeyPacket.decrypt/2`.\n\n#","ref":"OpenPGP.html"},{"type":"module","title":"Examples: - OpenPGP","doc":"Decode message packets and then cast\n\n    iex> message = <<160, 24, 2, 120, 156, 243, 72, 205, 201, 201, 215, 81, 8, 207, 47, 202, 73,\n    ...>     81, 84, 84, 4, 0, 40, 213, 4, 172>>\n    ...>\n    iex> packets = OpenPGP.list_packets(message)\n    [\n      %OpenPGP.Packet{\n        body: [\n          %OpenPGP.Packet.BodyChunk{\n            chunk_length: {:fixed, 24},\n            data: <<2, 120, 156, 243, 72, 205, 201, 201, 215, 81, 8, 207, 47, 202, 73, 81, 84,\n                84, 4, 0, 40, 213, 4, 172>>,\n            header_length: 1\n          }\n        ],\n        tag: %OpenPGP.Packet.PacketTag{\n          format: :old,\n          length_type: {0, \"one-octet\"},\n          tag: {8, \"Compressed Data Packet\"}\n        }\n      }\n    ]\n    iex> OpenPGP.cast_packets(packets)\n    [\n      %OpenPGP.CompressedDataPacket{\n        algo: {2, \"ZLIB [RFC1950]\"},\n        data_deflated: <<120, 156, 243, 72, 205, 201, 201, 215, 81, 8, 207, 47, 202, 73, 81, 84,\n            84, 4, 0, 40, 213, 4, 172>>,\n        data_inflated: \"Hello, World!!!\"\n      }\n    ]","ref":"OpenPGP.html#module-examples"},{"type":"function","title":"OpenPGP.cast_packet/1","doc":"Cast a generic packet %Packet{} to a speicific struct with a packet\nspecific data assigned.\n\n> NOTE: As of 0.5.x subset of RFC4880 Packet Tags can be casted. Other Packet\ntags remain as %Packet{} (not casted). Should not be considered as\nerror.","ref":"OpenPGP.html#cast_packet/1"},{"type":"function","title":"OpenPGP.cast_packets/1","doc":"Similar to `.cast_packet/1`, but operates on a list of generic\npackets.\n\n> NOTE: As of 0.5.x subset of RFC4880 Packet Tags can be casted. Other Packet\ntags remain as %Packet{} (not casted). Should not be considered as\nerror.","ref":"OpenPGP.html#cast_packets/1"},{"type":"function","title":"OpenPGP.list_packets/1","doc":"Decode all packets in a message (input).\nReturn a list of %Packet{} structs. Does not cast packets. To cast\ngeneric packets, use `.cast_packets/1` after `.list_packets/1`, i.e.\n<<...>> |> OpenPGP.list_packets() |> OpenPGP.cast_packets()\n\nThis function extremely handy for inspection, when operating at PTag\nand BodyChunk level.","ref":"OpenPGP.html#list_packets/1"},{"type":"type","title":"OpenPGP.any_packet/0","doc":"","ref":"OpenPGP.html#t:any_packet/0"},{"type":"behaviour","title":"OpenPGP.Packet.Behaviour","doc":"All packet specific decoders must implement\n`OpenPGP.Packet.Behaviour`, which exposes `.decode/1` interface\n(including genric `OpenPGP.Packet`). Additionaly some of the packet\nspecific decoders may provide interface for further packet processing,\nsuch as `OpenPGP.SecretKeyPacket.decrypt/2`.","ref":"OpenPGP.Packet.Behaviour.html"},{"type":"callback","title":"OpenPGP.Packet.Behaviour.decode/1","doc":"This callback is widely used to provide a clear interface to decoding\nOpenPGP packets. All tag-specific packets implement this callback:\naccepting a binary input and returning a two element tuple with a\ndecoded packet and a remainder of an input binary.","ref":"OpenPGP.Packet.Behaviour.html#c:decode/1"},{"type":"module","title":"OpenPGP.S2KSpecifier","doc":"Represents structured data for String-to-Key specifier.\n\n---\n\n## [RFC4880](https://www.ietf.org/rfc/rfc4880.txt)","ref":"OpenPGP.S2KSpecifier.html"},{"type":"module","title":"3.7.  String-to-Key (S2K) Specifiers - OpenPGP.S2KSpecifier","doc":"String-to-key (S2K) specifiers are used to convert passphrase strings\n into symmetric-key encryption/decryption keys.  They are used in two\n places, currently: to encrypt the secret part of private keys in the\n private keyring, and to convert passphrases to encryption keys for\n symmetrically encrypted messages.\n\n#","ref":"OpenPGP.S2KSpecifier.html#module-3-7-string-to-key-s2k-specifiers"},{"type":"module","title":"3.7.1.  String-to-Key (S2K) Specifier Types - OpenPGP.S2KSpecifier","doc":"There are three types of S2K specifiers currently supported, and\n some reserved values:\n\n     ID          S2K Type\n     --          --------\n     0           Simple S2K\n     1           Salted S2K\n     2           Reserved value\n     3           Iterated and Salted S2K\n     100 to 110  Private/Experimental S2K\n\n These are described in Sections 3.7.1.1 - 3.7.1.3.\n\n##","ref":"OpenPGP.S2KSpecifier.html#module-3-7-1-string-to-key-s2k-specifier-types"},{"type":"module","title":"3.7.1.1.  Simple S2K - OpenPGP.S2KSpecifier","doc":"This directly hashes the string to produce the key data.  See below\n for how this hashing is done.\n\n     Octet 0:        0x00\n     Octet 1:        hash algorithm\n\n Simple S2K hashes the passphrase to produce the session key.  The\n manner in which this is done depends on the size of the session key\n (which will depend on the cipher used) and the size of the hash\n algorithm's output.  If the hash size is greater than the session key\n size, the high-order (leftmost) octets of the hash are used as the\n key.\n\n If the hash size is less than the key size, multiple instances of the\n hash context are created -- enough to produce the required key data.\n These instances are preloaded with 0, 1, 2, ... octets of zeros (that\n is to say, the first instance has no preloading, the second gets\n preloaded with 1 octet of zero, the third is preloaded with two\n octets of zeros, and so forth).\n\n As the data is hashed, it is given independently to each hash\n context.  Since the contexts have been initialized differently, they\n will each produce different hash output.  Once the passphrase is\n hashed, the output data from the multiple hashes is concatenated,\n first hash leftmost, to produce the key data, with any excess octets\n on the right discarded.\n\n##","ref":"OpenPGP.S2KSpecifier.html#module-3-7-1-1-simple-s2k"},{"type":"module","title":"3.7.1.2.  Salted S2K - OpenPGP.S2KSpecifier","doc":"This includes a \"salt\" value in the S2K specifier -- some arbitrary\n data -- that gets hashed along with the passphrase string, to help\n prevent dictionary attacks.\n\n     Octet 0:        0x01\n     Octet 1:        hash algorithm\n     Octets 2-9:     8-octet salt value\n\n Salted S2K is exactly like Simple S2K, except that the input to the\n hash function(s) consists of the 8 octets of salt from the S2K\n specifier, followed by the passphrase.\n\n##","ref":"OpenPGP.S2KSpecifier.html#module-3-7-1-2-salted-s2k"},{"type":"module","title":"3.7.1.3.  Iterated and Salted S2K - OpenPGP.S2KSpecifier","doc":"This includes both a salt and an octet count.  The salt is combined\n with the passphrase and the resulting value is hashed repeatedly.\n This further increases the amount of work an attacker must do to try\n dictionary attacks.\n\n     Octet  0:        0x03\n     Octet  1:        hash algorithm\n     Octets 2-9:      8-octet salt value\n     Octet  10:       count, a one-octet, coded value\n The count is coded into a one-octet number using the following\n formula:\n\n     #define EXPBIAS 6\n         count = ((Int32)16 + (c & 15)) << ((c >> 4) + EXPBIAS);\n\n The above formula is in C, where \"Int32\" is a type for a 32-bit\n integer, and the variable \"c\" is the coded count, Octet 10.\n\n Iterated-Salted S2K hashes the passphrase and salt data multiple\n times.  The total number of octets to be hashed is specified in the\n encoded count in the S2K specifier.  Note that the resulting count\n value is an octet count of how many octets will be hashed, not an\n iteration count.\n\n Initially, one or more hash contexts are set up as with the other S2K\n algorithms, depending on how many octets of key data are needed.\n Then the salt, followed by the passphrase data, is repeatedly hashed\n until the number of octets specified by the octet count has been\n hashed.  The one exception is that if the octet count is less than\n the size of the salt plus passphrase, the full salt plus passphrase\n will be hashed even though that is greater than the octet count.\n After the hashing is done, the data is unloaded from the hash\n context(s) as with the other S2K algorithms.","ref":"OpenPGP.S2KSpecifier.html#module-3-7-1-3-iterated-and-salted-s2k"},{"type":"function","title":"OpenPGP.S2KSpecifier.build_session_key/3","doc":"","ref":"OpenPGP.S2KSpecifier.html#build_session_key/3"},{"type":"function","title":"OpenPGP.S2KSpecifier.decode/1","doc":"Decode String-to-Key specifier given input binary.\nReturn structured specifier and remaining binary.","ref":"OpenPGP.S2KSpecifier.html#decode/1"},{"type":"type","title":"OpenPGP.S2KSpecifier.t/0","doc":"","ref":"OpenPGP.S2KSpecifier.html#t:t/0"},{"type":"module","title":"OpenPGP.Util","doc":"Provides a set of utility functions to work with data.","ref":"OpenPGP.Util.html"},{"type":"function","title":"OpenPGP.Util.compression_algo_tuple/1","doc":"Convert compression algorithm ID to a tuple with ID and name binary.\n\n---\n\nRFC4880 (https://www.ietf.org/rfc/rfc4880.txt)\n\n9.3.  Compression Algorithms\n\n    ID           Algorithm\n    --           ---------\n    0          - Uncompressed\n    1          - ZIP [RFC1951]\n    2          - ZLIB [RFC1950]\n    3          - BZip2 [BZ2]\n    100 to 110 - Private/Experimental algorithm","ref":"OpenPGP.Util.html#compression_algo_tuple/1"},{"type":"function","title":"OpenPGP.Util.concat_body/1","doc":"Concatenates packet body given a Packet or a list of BodyChunks.","ref":"OpenPGP.Util.html#concat_body/1"},{"type":"function","title":"OpenPGP.Util.decode_mpi/1","doc":"Decode Multiprecision integer (MPI) given input binary.\nReturn MPI value and remaining binary.\n\n## [RFC4880](https://www.ietf.org/rfc/rfc4880.txt)\n\n#","ref":"OpenPGP.Util.html#decode_mpi/1"},{"type":"function","title":"3.2.  Multiprecision Integers - OpenPGP.Util.decode_mpi/1","doc":"Multiprecision integers (also called MPIs) are unsigned integers used\nto hold large integers such as the ones used in cryptographic\ncalculations.\n\nAn MPI consists of two pieces: a two-octet scalar that is the length\nof the MPI in bits followed by a string of octets that contain the\nactual integer.\n\nThese octets form a big-endian number; a big-endian number can be\nmade into an MPI by prefixing it with the appropriate length.\n\nExamples:\n\n(all numbers are in hexadecimal)\n\nThe string of octets [00 01 01] forms an MPI with the value 1.  The\nstring [00 09 01 FF] forms an MPI with the value of 511.\n\nAdditional rules:\n\nThe size of an MPI is ((MPI.length + 7) / 8) + 2 octets.\n\nThe length field of an MPI describes the length starting from its\nmost significant non-zero bit.  Thus, the MPI [00 02 01] is not\nformed correctly.  It should be [00 01 01].\n\nUnused bits of an MPI MUST be zero.\n\nAlso note that when an MPI is encrypted, the length refers to the\nplaintext MPI.  It may be ill-formed in its ciphertext.","ref":"OpenPGP.Util.html#decode_mpi/1-3-2-multiprecision-integers"},{"type":"function","title":"OpenPGP.Util.encode_mpi/1","doc":"Invers of `.decode_mpi/1`. Takes an MPI value, and encode it as MPI\nbinary.\n\n#","ref":"OpenPGP.Util.html#encode_mpi/1"},{"type":"function","title":"Example: - OpenPGP.Util.encode_mpi/1","doc":"iex> OpenPGP.Util.encode_mpi(<<0x1>>)\n    <<0, 0x1, 0x1>>\n\n    iex> OpenPGP.Util.encode_mpi(<<0x1, 0xFF>>)\n    <<0x0, 0x9, 0x1, 0xFF>>","ref":"OpenPGP.Util.html#encode_mpi/1-example"},{"type":"function","title":"OpenPGP.Util.public_key_algo_tuple/1","doc":"Convert public-key algorithm ID to a tuple with ID and name binary.\n\n---\n\nRFC4880 (https://www.ietf.org/rfc/rfc4880.txt)\n\n9.1.  Public-Key Algorithms\n+-----------+----------------------------------------------------+\n|        ID | Algorithm                                          |\n+-----------+----------------------------------------------------+\n|         1 | RSA (Encrypt or Sign) [HAC]                        |\n|         2 | RSA Encrypt-Only [HAC]                             |\n|         3 | RSA Sign-Only [HAC]                                |\n|        16 | Elgamal (Encrypt-Only) [ELGAMAL] [HAC]             |\n|        17 | DSA (Digital Signature Algorithm) [FIPS186] [HAC]  |\n|        18 | ECDH public key algorithm                          |\n|        19 | ECDSA public key algorithm [FIPS186]               |\n|        20 | Reserved (formerly Elgamal Encrypt or Sign)        |\n|        21 | Reserved for Diffie-Hellman                        |\n|           | (X9.42, as defined for IETF-S/MIME)                |\n|        22 | EdDSA [RFC8032]                                    |\n|        23 | Reserved for AEDH                                  |\n|        24 | Reserved for AEDSA                                 |\n|  100--110 | Private/Experimental algorithm                     |\n+-----------+----------------------------------------------------+","ref":"OpenPGP.Util.html#public_key_algo_tuple/1"},{"type":"function","title":"OpenPGP.Util.sym_algo_cipher_block_size/1","doc":"Detects cipher block size (bits) given symmetric encryption algorithm ID or a tuple.","ref":"OpenPGP.Util.html#sym_algo_cipher_block_size/1"},{"type":"function","title":"OpenPGP.Util.sym_algo_key_size/1","doc":"Detects cipher key size (bits) given symmetric encryption algorithm ID or a tuple.","ref":"OpenPGP.Util.html#sym_algo_key_size/1"},{"type":"function","title":"OpenPGP.Util.sym_algo_tuple/1","doc":"Convert symmetric encryption algorithm ID to a tuple with ID and name binary.\n\n---\n\nRFC4880 (https://www.ietf.org/rfc/rfc4880.txt)\n\n9.3.  Symmetric-Key Algorithms\n+-----------+-----------------------------------------------+\n|        ID | Algorithm                                     |\n+-----------+-----------------------------------------------+\n|         0 | Plaintext or unencrypted data                 |\n|         1 | IDEA [IDEA]                                   |\n|         2 | TripleDES (DES-EDE, [SCHNEIER] [HAC]          |\n|           | - 168 bit key derived from 192)               |\n|         3 | CAST5 (128 bit key, as per [RFC2144])         |\n|         4 | Blowfish (128 bit key, 16 rounds) [BLOWFISH]  |\n|         5 | Reserved                                      |\n|         6 | Reserved                                      |\n|         7 | AES with 128-bit key [AES]                    |\n|         8 | AES with 192-bit key                          |\n|         9 | AES with 256-bit key                          |\n|        10 | Twofish with 256-bit key [TWOFISH]            |\n|        11 | Camellia with 128-bit key [RFC3713]           |\n|        12 | Camellia with 192-bit key                     |\n|        13 | Camellia with 256-bit key                     |\n|  100--110 | Private/Experimental algorithm                |\n+-----------+-----------------------------------------------+","ref":"OpenPGP.Util.html#sym_algo_tuple/1"},{"type":"type","title":"OpenPGP.Util.compression_algo_tuple/0","doc":"","ref":"OpenPGP.Util.html#t:compression_algo_tuple/0"},{"type":"type","title":"OpenPGP.Util.public_key_algo_tuple/0","doc":"","ref":"OpenPGP.Util.html#t:public_key_algo_tuple/0"},{"type":"type","title":"OpenPGP.Util.sym_algo_tuple/0","doc":"","ref":"OpenPGP.Util.html#t:sym_algo_tuple/0"},{"type":"module","title":"OpenPGP.Packet","doc":"Packet struct represents a generic packet with a packet tag and a\nbody as a list of body chunks (see `OpenPGP.Packet.BodyChunk`). This\nabstraction layer operates at Packet Tag and Packet Body level only.\nTo interpret the contents of a packet, the packet body should be\ndecoded at another abstraction layer with packet tag-specific\ndecoders, for exampe `OpenPGP.LiteralDataPacket`\n\n---\n\n## [RFC4880](https://www.ietf.org/rfc/rfc4880.txt)\n\n#","ref":"OpenPGP.Packet.html"},{"type":"module","title":"4.1.  Overview - OpenPGP.Packet","doc":"An OpenPGP message is constructed from a number of records that are\ntraditionally called packets.  A packet is a chunk of data that has a\ntag specifying its meaning.  An OpenPGP message, keyring,\ncertificate, and so forth consists of a number of packets.  Some of\nthose packets may contain other OpenPGP packets (for example, a\ncompressed data packet, when uncompressed, contains OpenPGP packets).\n\nEach packet consists of a packet header, followed by the packet body.\nThe packet header is of variable length.","ref":"OpenPGP.Packet.html#module-4-1-overview"},{"type":"function","title":"OpenPGP.Packet.decode/1","doc":"Decode packet given input binary.\nReturn structured packet and remaining binary.\nExpect input to start with the Packet Tag octet.\n\n#","ref":"OpenPGP.Packet.html#decode/1"},{"type":"function","title":"Example: - OpenPGP.Packet.decode/1","doc":"iex> alias OpenPGP.Packet\n    iex> alias OpenPGP.Packet.PacketTag\n    iex> alias OpenPGP.Packet.BodyChunk\n    iex> Packet.decode(<<1::1, 0::1, 2::4, 0::2, 7::8, \"Hello, World!!!\">>)\n    {\n      %Packet{\n        tag: %PacketTag{format: :old, length_type: {0, \"one-octet\"}, tag: {2, \"Signature Packet\"}},\n        body: [%BodyChunk{chunk_length: {:fixed, 7}, data: \"Hello, \", header_length: 1}]\n      },\n      \"World!!!\"\n    }","ref":"OpenPGP.Packet.html#decode/1-example"},{"type":"type","title":"OpenPGP.Packet.t/0","doc":"","ref":"OpenPGP.Packet.html#t:t/0"},{"type":"module","title":"OpenPGP.Packet.BodyChunk","doc":"Packet data is represented as a list of body chunks. Each body\nchunk includes chunk length header information and chunk data.\n\nMost of the packets will have only one body chunk. Only packets with\nPartial Body Length expected to have more than one body chunk.\n\nThe `OpenPGP.Util.concat_body/1` can be applied to a packet body or a\nlist of body chunks to concatenate chunks into a binary to be further\ninterpreted.\n---\n\n## [RFC4880](https://www.ietf.org/rfc/rfc4880.txt)\n\n##","ref":"OpenPGP.Packet.BodyChunk.html"},{"type":"module","title":"4.2.1.  Old Format Packet Lengths - OpenPGP.Packet.BodyChunk","doc":"The meaning of the length-type in old format packets is:\n\n0 - The packet has a one-octet length.  The header is 2 octets long.\n\n1 - The packet has a two-octet length.  The header is 3 octets long.\n\n2 - The packet has a four-octet length.  The header is 5 octets long.\n\n3 - The packet is of indeterminate length.  The header is 1 octet\n    long, and the implementation must determine how long the packet\n    is.  If the packet is in a file, this means that the packet\n    extends until the end of the file.  In general, an implementation\n    SHOULD NOT use indeterminate-length packets except where the end\n    of the data will be clear from the context, and even then it is\n    better to use a definite length, or a new format header.  The new\n    format headers described below have a mechanism for precisely\n    encoding data of indeterminate length.\n\n#","ref":"OpenPGP.Packet.BodyChunk.html#module-4-2-1-old-format-packet-lengths"},{"type":"module","title":"4.2.2.  New Format Packet Lengths - OpenPGP.Packet.BodyChunk","doc":"New format packets have four possible ways of encoding length:\n\n1. A one-octet Body Length header encodes packet lengths of up to 191\n  octets.\n\n2. A two-octet Body Length header encodes packet lengths of 192 to\n  8383 octets.\n\n3. A five-octet Body Length header encodes packet lengths of up to\n  4,294,967,295 (0xFFFFFFFF) octets in length.  (This actually\n  encodes a four-octet scalar number.)\n\n4. When the length of the packet body is not known in advance by the\n  issuer, Partial Body Length headers encode a packet of\n  indeterminate length, effectively making it a stream.\n\n##","ref":"OpenPGP.Packet.BodyChunk.html#module-4-2-2-new-format-packet-lengths"},{"type":"module","title":"4.2.2.1.  One-Octet Lengths - OpenPGP.Packet.BodyChunk","doc":"A one-octet Body Length header encodes a length of 0 to 191 octets.\nThis type of length header is recognized because the one octet value\nis less than 192.  The body length is equal to:\n\n    bodyLen = 1st_octet;\n\n##","ref":"OpenPGP.Packet.BodyChunk.html#module-4-2-2-1-one-octet-lengths"},{"type":"module","title":"4.2.2.2.  Two-Octet Lengths - OpenPGP.Packet.BodyChunk","doc":"A two-octet Body Length header encodes a length of 192 to 8383\noctets.  It is recognized because its first octet is in the range 192\nto 223.  The body length is equal to:\n\n    bodyLen = ((1st_octet - 192) << 8) + (2nd_octet) + 192\n\n##","ref":"OpenPGP.Packet.BodyChunk.html#module-4-2-2-2-two-octet-lengths"},{"type":"module","title":"4.2.2.3.  Five-Octet Lengths - OpenPGP.Packet.BodyChunk","doc":"A five-octet Body Length header consists of a single octet holding\nthe value 255, followed by a four-octet scalar.  The body length is\nequal to:\n\n    bodyLen = (2nd_octet << 24) | (3rd_octet << 16) |\n              (4th_octet << 8)  | 5th_octet\n\nThis basic set of one, two, and five-octet lengths is also used\ninternally to some packets.\n\n##","ref":"OpenPGP.Packet.BodyChunk.html#module-4-2-2-3-five-octet-lengths"},{"type":"module","title":"4.2.2.4.  Partial Body Lengths - OpenPGP.Packet.BodyChunk","doc":"A Partial Body Length header is one octet long and encodes the length\nof only part of the data packet.  This length is a power of 2, from 1\nto 1,073,741,824 (2 to the 30th power).  It is recognized by its one\noctet value that is greater than or equal to 224, and less than 255.\nThe Partial Body Length is equal to:\n\n    partialBodyLen = 1 << (1st_octet & 0x1F);\n\nEach Partial Body Length header is followed by a portion of the\npacket body data.  The Partial Body Length header specifies this\nportion's length.  Another length header (one octet, two-octet,\nfive-octet, or partial) follows that portion.  The last length header\nin the packet MUST NOT be a Partial Body Length header.  Partial Body\nLength headers may only be used for the non-final parts of the\npacket.\n\nNote also that the last Body Length header can be a zero-length\nheader.\n\nAn implementation MAY use Partial Body Lengths for data packets, be\nthey literal, compressed, or encrypted.  The first partial length\nMUST be at least 512 octets long.  Partial Body Lengths MUST NOT be\nused for any other packet types.","ref":"OpenPGP.Packet.BodyChunk.html#module-4-2-2-4-partial-body-lengths"},{"type":"function","title":"OpenPGP.Packet.BodyChunk.decode/2","doc":"Decode body chunk given input binary and packet tag.\nReturn structured body chunk and remaining binary.\nExpect input to start with Body Length header octets.\n\n#","ref":"OpenPGP.Packet.BodyChunk.html#decode/2"},{"type":"function","title":"Example: - OpenPGP.Packet.BodyChunk.decode/2","doc":"iex> alias OpenPGP.Packet.{BodyChunk, PacketTag}\n    iex> BodyChunk.decode(<<5, \"Hello\", \" world!\">>, %PacketTag{format: :old, length_type: {0, \"one-octet\"}})\n    {%BodyChunk{data: \"Hello\", header_length: 1, chunk_length: {:fixed, 5}}, \" world!\"}","ref":"OpenPGP.Packet.BodyChunk.html#decode/2-example"},{"type":"type","title":"OpenPGP.Packet.BodyChunk.chunk_length/0","doc":"Chunk length in octets (bytes). Payload size.","ref":"OpenPGP.Packet.BodyChunk.html#t:chunk_length/0"},{"type":"type","title":"OpenPGP.Packet.BodyChunk.header_length/0","doc":"The packet has a N-octet length header.","ref":"OpenPGP.Packet.BodyChunk.html#t:header_length/0"},{"type":"type","title":"OpenPGP.Packet.BodyChunk.t/0","doc":"","ref":"OpenPGP.Packet.BodyChunk.html#t:t/0"},{"type":"module","title":"OpenPGP.Packet.PacketTag","doc":"PacketTag struct represents a packet tag as per RFC4880.\n\n---\n\n## [RFC4880](https://www.ietf.org/rfc/rfc4880.txt)\n\n#","ref":"OpenPGP.Packet.PacketTag.html"},{"type":"module","title":"4.2.  Packet Headers - OpenPGP.Packet.PacketTag","doc":"The first octet of the packet header is called the \"Packet Tag\".  It\ndetermines the format of the header and denotes the packet contents.\nNote that the most significant bit is the leftmost bit, called bit 7.\nA mask for this bit is 0x80 in hexadecimal.\n\n          +---------------+\n      PTag |7 6 5 4 3 2 1 0|\n          +---------------+\n      Bit 7 -- Always one\n      Bit 6 -- New packet format if set\n\nNote that old format packets have four bits of\npacket tags, and new format packets have six; some features cannot be\nused and still be backward-compatible.\n\nAlso note that packets with a tag greater than or equal to 16 MUST\nuse new format packets.  The old format packets can only express tags\nless than or equal to 15.\n\nOld format packets contain:\n\n      Bits 5-2 -- packet tag\n      Bits 1-0 -- length-type\n\nNew format packets contain:\n\n      Bits 5-0 -- packet tag\n\n##","ref":"OpenPGP.Packet.PacketTag.html#module-4-2-packet-headers"},{"type":"module","title":"4.2.1.  Old Format Packet Lengths - OpenPGP.Packet.PacketTag","doc":"The meaning of the length-type in old format packets is:\n\n0 - The packet has a one-octet length.  The header is 2 octets long.\n\n1 - The packet has a two-octet length.  The header is 3 octets long.\n\n2 - The packet has a four-octet length.  The header is 5 octets long.\n\n3 - The packet is of indeterminate length.  The header is 1 octet\n    long, and the implementation must determine how long the packet\n    is.  If the packet is in a file, this means that the packet\n    extends until the end of the file.  In general, an implementation\n    SHOULD NOT use indeterminate-length packets except where the end\n    of the data will be clear from the context, and even then it is\n    better to use a definite length, or a new format header.  The new\n    format headers described below have a mechanism for precisely\n    encoding data of indeterminate length.\n\n#","ref":"OpenPGP.Packet.PacketTag.html#module-4-2-1-old-format-packet-lengths"},{"type":"module","title":"4.3.  Packet Tags - OpenPGP.Packet.PacketTag","doc":"The packet tag denotes what type of packet the body holds.  Note that\nold format headers can only have tags less than 16, whereas new\nformat headers can have tags as great as 63.  The defined tags (in\ndecimal) are as follows:\n\n    0        -- Reserved - a packet tag MUST NOT have this value\n    1        -- Public-Key Encrypted Session Key Packet\n    2        -- Signature Packet\n    3        -- Symmetric-Key Encrypted Session Key Packet\n    4        -- One-Pass Signature Packet\n    5        -- Secret-Key Packet\n    6        -- Public-Key Packet\n    7        -- Secret-Subkey Packet\n    8        -- Compressed Data Packet\n    9        -- Symmetrically Encrypted Data Packet\n    10       -- Marker Packet\n    11       -- Literal Data Packet\n    12       -- Trust Packet\n    13       -- User ID Packet\n    14       -- Public-Subkey Packet\n    17       -- User Attribute Packet\n    18       -- Sym. Encrypted and Integrity Protected Data Packet\n    19       -- Modification Detection Code Packet\n    60 to 63 -- Private or Experimental Values","ref":"OpenPGP.Packet.PacketTag.html#module-4-3-packet-tags"},{"type":"function","title":"OpenPGP.Packet.PacketTag.decode/1","doc":"Decode packet tag given input binary.\nReturn structured packet tag and remaining binary.\nExpect input to start with the Packet Tag octet.\n\n#","ref":"OpenPGP.Packet.PacketTag.html#decode/1"},{"type":"function","title":"Example: - OpenPGP.Packet.PacketTag.decode/1","doc":"iex> alias OpenPGP.Packet.PacketTag\n    iex> PacketTag.decode(<<1::1, 0::1, 2::4, 0::2, \"data\">>)\n    {%PacketTag{format: :old, tag: {2, \"Signature Packet\"}, length_type: {0, \"one-octet\"}}, \"data\"}","ref":"OpenPGP.Packet.PacketTag.html#decode/1-example"},{"type":"type","title":"OpenPGP.Packet.PacketTag.length_tuple/0","doc":"","ref":"OpenPGP.Packet.PacketTag.html#t:length_tuple/0"},{"type":"type","title":"OpenPGP.Packet.PacketTag.t/0","doc":"","ref":"OpenPGP.Packet.PacketTag.html#t:t/0"},{"type":"type","title":"OpenPGP.Packet.PacketTag.tag_tuple/0","doc":"","ref":"OpenPGP.Packet.PacketTag.html#t:tag_tuple/0"},{"type":"module","title":"OpenPGP.CompressedDataPacket","doc":"Represents structured data for Compressed Data Packet.\n\n#","ref":"OpenPGP.CompressedDataPacket.html"},{"type":"module","title":"Example: - OpenPGP.CompressedDataPacket","doc":"iex> alias OpenPGP.CompressedDataPacket\n    iex> deflated = <<120, 156, 243, 72, 205, 201, 201, 215, 81, 8,\n    ...>     207, 47, 202, 73, 81, 84, 84, 4, 0, 40, 213, 4, 172>>\n    iex> CompressedDataPacket.decode(<<2, deflated::binary>>)\n    {\n      %CompressedDataPacket{\n        algo: {2, \"ZLIB [RFC1950]\"},\n        data_deflated: deflated,\n        data_inflated: \"Hello, World!!!\"},\n      <<>>\n    }\n\n> NOTE: As of 0.5.x Compressed Data Packet supports only:\n\n  1. ZIP-style blocks (ID: 1)\n  1. ZLIB-style blocks (ID: 2)\n\n\n---\n\n## [RFC4880](https://www.ietf.org/rfc/rfc4880.txt)\n\n#","ref":"OpenPGP.CompressedDataPacket.html#module-example"},{"type":"module","title":"5.6.  Compressed Data Packet (Tag 8) - OpenPGP.CompressedDataPacket","doc":"The Compressed Data packet contains compressed data.  Typically, this\npacket is found as the contents of an encrypted packet, or following\na Signature or One-Pass Signature packet, and contains a literal data\npacket.\n\nThe body of this packet consists of:\n\n  - One octet that gives the algorithm used to compress the packet.\n\n  - Compressed data, which makes up the remainder of the packet.\n\nA Compressed Data Packet's body contains an block that compresses\nsome set of packets.  See section \"Packet Composition\" for details on\nhow messages are formed.\n\nZIP-compressed packets are compressed with raw RFC 1951 [RFC1951]\nDEFLATE blocks.  Note that PGP V2.6 uses 13 bits of compression.  If\nan implementation uses more bits of compression, PGP V2.6 cannot\ndecompress it.\n\nZLIB-compressed packets are compressed with RFC 1950 [RFC1950] ZLIB-\nstyle blocks.\n\nBZip2-compressed packets are compressed using the BZip2 [BZ2]\nalgorithm.","ref":"OpenPGP.CompressedDataPacket.html#module-5-6-compressed-data-packet-tag-8"},{"type":"function","title":"OpenPGP.CompressedDataPacket.decode/1","doc":"Decode Compressed Data Packet given input binary.\nReturn structured packet and remaining binary (empty binary).","ref":"OpenPGP.CompressedDataPacket.html#decode/1"},{"type":"type","title":"OpenPGP.CompressedDataPacket.t/0","doc":"","ref":"OpenPGP.CompressedDataPacket.html#t:t/0"},{"type":"module","title":"OpenPGP.IntegrityProtectedDataPacket","doc":"Represents structured data for Integrity Protected Data Packet.\n\n#","ref":"OpenPGP.IntegrityProtectedDataPacket.html"},{"type":"module","title":"Example: - OpenPGP.IntegrityProtectedDataPacket","doc":"iex> alias OpenPGP.IntegrityProtectedDataPacket\n    iex> alias OpenPGP.PublicKeyEncryptedSessionKeyPacket\n    ...>\n    iex> key = <<38, 165, 130, 172, 168, 51, 184, 238, 96, 204, 88,\n    ...>   134, 93, 25, 162, 22, 83, 211, 140, 176, 115, 113, 37, 201,\n    ...>   171, 249, 115, 64, 94, 59, 35, 60>>\n    ...>\n    iex> iv = <<0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0>>\n    iex> < > = :crypto.strong_rand_bytes(16)\n    iex> plaintext = < >\n    ...>\n    iex> ciphertext =\n    ...>   :crypto.crypto_one_time(\n    ...>    :aes_256_cfb128,\n    ...>    key,\n    ...>    iv,\n    ...>    plaintext,\n    ...>    true)\n    ...>\n    iex> payload = <<1::8, ciphertext::binary>>\n    iex> {packet_decoded, <<>>} =\n    ...>   IntegrityProtectedDataPacket.decode(payload)\n    {\n      %IntegrityProtectedDataPacket{\n        ciphertext: ciphertext,\n        plaintext: nil,\n        version: 1\n      },\n      <<>>\n    }\n    iex> pkesk = %PublicKeyEncryptedSessionKeyPacket{\n    ...>   version: 3,\n    ...>   session_key_algo: {9, \"AES with 256-bit key\"},\n    ...>   session_key_material: {key}\n    ...> }\n    ...>\n    iex> IntegrityProtectedDataPacket.decrypt(packet_decoded, pkesk)\n    %IntegrityProtectedDataPacket{\n      version: 1,\n      plaintext: \"Hello\",\n      ciphertext: ciphertext\n    }\n\n\n> NOTE: As of 0.5.x Symmetrically Encrypted Integrity Protected Data Packet:\n\n  1. Supports Session Key algo 9 (AES with 256-bit key) in CFB mode\n  2. Modification Detection Code system is not supported\n\n\n---\n\n## [RFC4880](https://www.ietf.org/rfc/rfc4880.txt)\n\n#","ref":"OpenPGP.IntegrityProtectedDataPacket.html#module-example"},{"type":"module","title":"5.13.  Sym. Encrypted Integrity Protected Data Packet (Tag 18) - OpenPGP.IntegrityProtectedDataPacket","doc":"The Symmetrically Encrypted Integrity Protected Data packet is a\nvariant of the Symmetrically Encrypted Data packet.\n\n...\n\nThis packet contains data encrypted with a symmetric-key algorithm\nand protected against modification by the SHA-1 hash algorithm.  When\nit has been decrypted, it will typically contain other packets (often\na Literal Data packet or Compressed Data packet).  The last decrypted\npacket in this packet's payload MUST be a Modification Detection Code\npacket.\n\nThe body of this packet consists of:\n\n  - A one-octet version number.  The only currently defined value is\n    1.\n\n  - Encrypted data, the output of the selected symmetric-key cipher\n    operating in Cipher Feedback mode with shift amount equal to the\n    block size of the cipher (CFB-n where n is the block size).\n\nThe symmetric cipher used MUST be specified in a Public-Key or\nSymmetric-Key Encrypted Session Key packet that precedes the\nSymmetrically Encrypted Data packet.  In either case, the cipher\nalgorithm octet is prefixed to the session key before it is\nencrypted.\n\nThe data is encrypted in CFB mode, with a CFB shift size equal to the\ncipher's block size.  The Initial Vector (IV) is specified as all\nzeros.  Instead of using an IV, OpenPGP prefixes an octet string to\nthe data before it is encrypted.  The length of the octet string\nequals the block size of the cipher in octets, plus two.  The first\noctets in the group, of length equal to the block size of the cipher,\nare random; the last two octets are each copies of their 2nd\npreceding octet.  For example, with a cipher whose block size is 128\nbits or 16 octets, the prefix data will contain 16 random octets,\nthen two more octets, which are copies of the 15th and 16th octets,\nrespectively.  Unlike the Symmetrically Encrypted Data Packet, no\nspecial CFB resynchronization is done after encrypting this prefix\ndata.  See \"OpenPGP CFB Mode\" below for more details.\n\nThe repetition of 16 bits in the random data prefixed to the message\nallows the receiver to immediately check whether the session key is\nincorrect.\n\n...","ref":"OpenPGP.IntegrityProtectedDataPacket.html#module-5-13-sym-encrypted-integrity-protected-data-packet-tag-18"},{"type":"function","title":"OpenPGP.IntegrityProtectedDataPacket.decode/1","doc":"Decode Sym. Encrypted and Integrity Protected Data Packet given input\nbinary.\nReturn structured packet and remaining binary (empty binary).","ref":"OpenPGP.IntegrityProtectedDataPacket.html#decode/1"},{"type":"function","title":"OpenPGP.IntegrityProtectedDataPacket.decrypt/2","doc":"Decrypt Sym. Encrypted and Integrity Protected Data Packet\n(PKESK-Packet) given decoded PKESK-Packet and a decrypted Public-Key\nEncrypted Session Key Packet.\nReturn PKESK-Packet with `:plaintext` attr assigned.\nRaises an error if checksum does not match.","ref":"OpenPGP.IntegrityProtectedDataPacket.html#decrypt/2"},{"type":"type","title":"OpenPGP.IntegrityProtectedDataPacket.t/0","doc":"","ref":"OpenPGP.IntegrityProtectedDataPacket.html#t:t/0"},{"type":"module","title":"OpenPGP.LiteralDataPacket","doc":"Represents structured data for Literal Data Packet.\n\n#","ref":"OpenPGP.LiteralDataPacket.html"},{"type":"module","title":"Example: - OpenPGP.LiteralDataPacket","doc":"iex> alias OpenPGP.LiteralDataPacket\n    ...> data = <<0x62, 11, \"example.txt\", 1704328052::32, \"Hello!\">>\n    ...> LiteralDataPacket.decode(data)\n    {\n      %LiteralDataPacket{\n        created_at: ~U[2024-01-04 00:27:32Z],\n        data: \"Hello!\",\n        file_name: \"example.txt\",\n        format: {<<0x62>>, :binary}\n      },\n      <<>>\n    }\n\n---\n\n## [RFC4880](https://www.ietf.org/rfc/rfc4880.txt)\n\n#","ref":"OpenPGP.LiteralDataPacket.html#module-example"},{"type":"module","title":"5.9.  Literal Data Packet (Tag 11) - OpenPGP.LiteralDataPacket","doc":"A Literal Data packet contains the body of a message; data that is\nnot to be further interpreted.\n\nThe body of this packet consists of:\n\n  - A one-octet field that describes how the data is formatted.\n\nIf it is a 'b' (0x62), then the Literal packet contains binary data.\nIf it is a 't' (0x74), then it contains text data, and thus may need\nline ends converted to local form, or other text-mode changes.  The\ntag 'u' (0x75) means the same as 't', but also indicates that\nimplementation believes that the literal data contains UTF-8 text.\n\nEarly versions of PGP also defined a value of 'l' as a 'local' mode\nfor machine-local conversions.  RFC 1991 [RFC1991] incorrectly stated\nthis local mode flag as '1' (ASCII numeral one).  Both of these local\nmodes are deprecated.\n\n  - File name as a string (one-octet length, followed by a file\n    name).  This may be a zero-length string.  Commonly, if the\n    source of the encrypted data is a file, this will be the name of\n    the encrypted file.  An implementation MAY consider the file name\n    in the Literal packet to be a more authoritative name than the\n    actual file name.\n\nIf the special name \"_CONSOLE\" is used, the message is considered to\nbe \"for your eyes only\".  This advises that the message data is\nunusually sensitive, and the receiving program should process it more\ncarefully, perhaps avoiding storing the received data to disk, for\nexample.\n\n  - A four-octet number that indicates a date associated with the\n    literal data.  Commonly, the date might be the modification date\n    of a file, or the time the packet was created, or a zero that\n    indicates no specific time.\n\n  - The remainder of the packet is literal data.\n\n    Text data is stored with    text endings (i.e., network-\n    normal line endings).  These should be converted to native line\n    endings by the receiving software.","ref":"OpenPGP.LiteralDataPacket.html#module-5-9-literal-data-packet-tag-11"},{"type":"function","title":"OpenPGP.LiteralDataPacket.decode/1","doc":"Decode Literal Data Packet given input binary.\nReturn structured packet and remaining binary (empty binary).","ref":"OpenPGP.LiteralDataPacket.html#decode/1"},{"type":"type","title":"OpenPGP.LiteralDataPacket.t/0","doc":"","ref":"OpenPGP.LiteralDataPacket.html#t:t/0"},{"type":"module","title":"OpenPGP.PublicKeyEncryptedSessionKeyPacket","doc":"Represents structured data for Public-Key Encrypted SessionKey Packet.\n\nThe `:ciphertext` attribute is set once the packet is decoded with\n`.decode/1` and the packet data is still symmetrically encrypted. The\nnext logical step is to decrypt packet with `.decrypt/2` to get\nsymmetrically encrypted session key material.\n\n---\n\n## [RFC4880](https://www.ietf.org/rfc/rfc4880.txt)\n\n#","ref":"OpenPGP.PublicKeyEncryptedSessionKeyPacket.html"},{"type":"module","title":"5.1.  Public-Key Encrypted Session Key Packets (Tag 1) - OpenPGP.PublicKeyEncryptedSessionKeyPacket","doc":"A Public-Key Encrypted Session Key packet holds the session key used\nto encrypt a message.  Zero or more Public-Key Encrypted Session Key\npackets and/or Symmetric-Key Encrypted Session Key packets may\nprecede a Symmetrically Encrypted Data Packet, which holds an\nencrypted message.  The message is encrypted with the session key,\nand the session key is itself encrypted and stored in the Encrypted\nSession Key packet(s).  The Symmetrically Encrypted Data Packet is\npreceded by one Public-Key Encrypted Session Key packet for each\nOpenPGP key to which the message is encrypted.  The recipient of the\nmessage finds a session key that is encrypted to their public key,\ndecrypts the session key, and then uses the session key to decrypt\nthe message.\n\nThe body of this packet consists of:\n\n  - A one-octet number giving the version number of the packet type.\n    The currently defined value for packet version is 3.\n\n  - An eight-octet number that gives the Key ID of the public key to\n    which the session key is encrypted.  If the session key is\n    encrypted to a subkey, then the Key ID of this subkey is used\n    here instead of the Key ID of the primary key.\n\n  - A one-octet number giving the public-key algorithm used.\n\n  - A string of octets that is the encrypted session key.  This\n    string takes up the remainder of the packet, and its contents are\n    dependent on the public-key algorithm used.\n\nAlgorithm Specific Fields for RSA encryption\n\n  - multiprecision integer (MPI) of RSA encrypted value m**e mod n.\n\nAlgorithm Specific Fields for Elgamal encryption:\n\n  - MPI of Elgamal (Diffie-Hellman) value g**k mod p.\n\n  - MPI of Elgamal (Diffie-Hellman) value m * y**k mod p.\n\nThe value \"m\" in the above formulas is derived from the session key\nas follows.  First, the session key is prefixed with a one-octet\nalgorithm identifier that specifies the symmetric encryption\nalgorithm used to encrypt the following Symmetrically Encrypted Data\nPacket.  Then a two-octet checksum is appended, which is equal to the\nsum of the preceding session key octets, not including the algorithm\nidentifier, modulo 65536.  This value is then encoded as described in\nPKCS#1 block encoding EME-PKCS1-v1_5 in Section 7.2.1 of [RFC3447] to\nform the \"m\" value used in the formulas above.  See Section 13.1 of\nthis document for notes on OpenPGP's use of PKCS#1.\n\nNote that when an implementation forms several PKESKs with one\nsession key, forming a message that can be decrypted by several keys,\nthe implementation MUST make a new PKCS#1 encoding for each key.\n\nAn implementation MAY accept or use a Key ID of zero as a \"wild card\"\nor \"speculative\" Key ID.  In this case, the receiving implementation\nwould try all available private keys, checking for a valid decrypted\nsession key.  This format helps reduce traffic analysis of messages.","ref":"OpenPGP.PublicKeyEncryptedSessionKeyPacket.html#module-5-1-public-key-encrypted-session-key-packets-tag-1"},{"type":"function","title":"OpenPGP.PublicKeyEncryptedSessionKeyPacket.decode/1","doc":"Decode Public-Key Encrypted SessionKey Packet given input binary.\nReturn structured packet and remaining binary.","ref":"OpenPGP.PublicKeyEncryptedSessionKeyPacket.html#decode/1"},{"type":"function","title":"OpenPGP.PublicKeyEncryptedSessionKeyPacket.decrypt/2","doc":"Decrypt Public-Key Encrypted Session Key Packet given decoded\nPublic-Key Encrypted Session Key Packet and decoded and decrypted\nSecret-Key Packet.\nReturn Public-Key Encrypted Session Key Packet with\n`:session_key_algo` and `:session_key_material` attrs assigned.\nRaises an error if checksum does not match.","ref":"OpenPGP.PublicKeyEncryptedSessionKeyPacket.html#decrypt/2"},{"type":"type","title":"OpenPGP.PublicKeyEncryptedSessionKeyPacket.t/0","doc":"","ref":"OpenPGP.PublicKeyEncryptedSessionKeyPacket.html#t:t/0"},{"type":"module","title":"OpenPGP.PublicKeyPacket","doc":"Represents structured data for Public-Key Packet.\n\n---\n\n## [RFC4880](https://www.ietf.org/rfc/rfc4880.txt)\n\n#","ref":"OpenPGP.PublicKeyPacket.html"},{"type":"module","title":"5.5.2.  Public-Key Packet Formats - OpenPGP.PublicKeyPacket","doc":"There are two versions of key-material packets.  Version 3 packets\nwere first generated by PGP 2.6.  Version 4 keys first appeared in\nPGP 5.0 and are the preferred key version for OpenPGP.\n\nOpenPGP implementations MUST create keys with version 4 format.  V3\nkeys are deprecated; an implementation MUST NOT generate a V3 key,\nbut MAY accept it.\n\nA version 3 public key or public-subkey packet contains:\n\n  - A one-octet version number (3).\n\n  - A four-octet number denoting the time that the key was created.\n\n  - A two-octet number denoting the time in days that this key is\n    valid.  If this number is zero, then it does not expire.\n\n  - A one-octet number denoting the public-key algorithm of this key.\n\n  - A series of multiprecision integers comprising the key material:\n\n        - a multiprecision integer (MPI) of RSA public modulus n;\n\n        - an MPI of RSA public encryption exponent e.\n\nV3 keys are deprecated.  They contain three weaknesses.  First, it is\nrelatively easy to construct a V3 key that has the same Key ID as any\nother key because the Key ID is simply the low 64 bits of the public\nmodulus.  Secondly, because the fingerprint of a V3 key hashes the\nkey material, but not its length, there is an increased opportunity\nfor fingerprint collisions.  Third, there are weaknesses in the MD5\nhash algorithm that make developers prefer other algorithms.  See\nbelow for a fuller discussion of Key IDs and fingerprints.\n\nV2 keys are identical to the deprecated V3 keys except for the\nversion number.  An implementation MUST NOT generate them and MAY\naccept or reject them as it sees fit.\n\nThe version 4 format is similar to the version 3 format except for\nthe absence of a validity period.  This has been moved to the\nSignature packet.  In addition, fingerprints of version 4 keys are\ncalculated differently from version 3 keys, as described in the\nsection \"Enhanced Key Formats\".\n\nA version 4 packet contains:\n\n  - A one-octet version number (4).\n\n  - A four-octet number denoting the time that the key was created.\n\n  - A one-octet number denoting the public-key algorithm of this key.\n\n  - A series of multiprecision integers comprising the key material.\n    This algorithm-specific portion is:\n\n    Algorithm-Specific Fields for RSA public keys:\n\n      - multiprecision integer (MPI) of RSA public modulus n;\n\n      - MPI of RSA public encryption exponent e.\n\n    Algorithm-Specific Fields for DSA public keys:\n\n      - MPI of DSA prime p;\n\n      - MPI of DSA group order q (q is a prime divisor of p-1);\n\n      - MPI of DSA group generator g;\n\n      - MPI of DSA public-key value y (= g**x mod p where x\n        is secret).\n\n    Algorithm-Specific Fields for Elgamal public keys:\n\n      - MPI of Elgamal prime p;\n\n      - MPI of Elgamal group generator g;\n      - MPI of Elgamal public key value y (= g**x mod p where x\n        is secret).","ref":"OpenPGP.PublicKeyPacket.html#module-5-5-2-public-key-packet-formats"},{"type":"function","title":"OpenPGP.PublicKeyPacket.decode/1","doc":"Decode Public Key Packet given input binary.\nReturn structured packet and remaining binary.","ref":"OpenPGP.PublicKeyPacket.html#decode/1"},{"type":"type","title":"OpenPGP.PublicKeyPacket.t/0","doc":"","ref":"OpenPGP.PublicKeyPacket.html#t:t/0"},{"type":"module","title":"OpenPGP.SecretKeyPacket","doc":"Represents structured data for Secret-Key Packet.\n\n> NOTE: As of 0.5.x Secret-Key Packet supports only:\n\n  1. V4 packets\n  1. Iterated and Salted String-to-Key (S2K) specifier (ID: 3)\n  1. S2K usage convention octet of 254 only\n  1. S2K hashing algo SHA1\n  1. AES128 symmetric encryption of secret key material\n\n---\n\n## [RFC4880](https://www.ietf.org/rfc/rfc4880.txt)\n\n5.5.3.  Secret-Key Packet Formats\n\nThe Secret-Key and Secret-Subkey packets contain all the data of the\nPublic-Key and Public-Subkey packets, with additional algorithm-\nspecific secret-key data appended, usually in encrypted form.\n\nThe packet contains:\n\n  - A Public-Key or Public-Subkey packet, as described above.\n\n  - One octet indicating string-to-key usage conventions.  Zero\n    indicates that the secret-key data is not encrypted.  255 or 254\n    indicates that a string-to-key specifier is being given.  Any\n    other value is a symmetric-key encryption algorithm identifier.\n\n  - [Optional] If string-to-key usage octet was 255 or 254, a one-\n    octet symmetric encryption algorithm.\n\n  - [Optional] If string-to-key usage octet was 255 or 254, a\n    string-to-key specifier.  The length of the string-to-key\n    specifier is implied by its type, as described above.\n\n  - [Optional] If secret data is encrypted (string-to-key usage octet\n    not zero), an Initial Vector (IV) of the same length as the\n    cipher's block size.\n\n  - Plain or encrypted multiprecision integers comprising the secret\n    key data.  These algorithm-specific fields are as described\n    below.\n\n  - If the string-to-key usage octet is zero or 255, then a two-octet\n    checksum of the plaintext of the algorithm-specific portion (sum\n    of all octets, mod 65536).  If the string-to-key usage octet was\n    254, then a 20-octet SHA-1 hash of the plaintext of the\n    algorithm-specific portion.  This checksum or hash is encrypted\n    together with the algorithm-specific fields (if string-to-key\n    usage octet is not zero).  Note that for all other values, a\n    two-octet checksum is required.\n\n    Algorithm-Specific Fields for RSA secret keys:\n\n    - multiprecision integer (MPI) of RSA secret exponent d.\n\n    - MPI of RSA secret prime value p.\n\n    - MPI of RSA secret prime value q (p < q).\n\n    - MPI of u, the multiplicative inverse of p, mod q.\n\n    Algorithm-Specific Fields for DSA secret keys:\n\n    - MPI of DSA secret exponent x.\n\n    Algorithm-Specific Fields for Elgamal secret keys:\n\n    - MPI of Elgamal secret exponent x.\n\nSecret MPI values can be encrypted using a passphrase.  If a string-\nto-key specifier is given, that describes the algorithm for\nconverting the passphrase to a key, else a simple MD5 hash of the\npassphrase is used.  Implementations MUST use a string-to-key\nspecifier; the simple hash is for backward compatibility and is\ndeprecated, though implementations MAY continue to use existing\nprivate keys in the old format.  The cipher for encrypting the MPIs\nis specified in the Secret-Key packet.\n\nEncryption/decryption of the secret data is done in CFB mode using\nthe key created from the passphrase and the Initial Vector from the\npacket.  A different mode is used with V3 keys (which are only RSA)\nthan with other key formats.  With V3 keys, the MPI bit count prefix\n(i.e., the first two octets) is not encrypted.  Only the MPI non-\nprefix data is encrypted.  Furthermore, the CFB state is\nresynchronized at the beginning of each new MPI value, so that the\nCFB block boundary is aligned with the start of the MPI data.\n\nWith V4 keys, a simpler method is used.  All secret MPI values are\nencrypted in CFB mode, including the MPI bitcount prefix.\n\nThe two-octet checksum that follows the algorithm-specific portion is\nthe algebraic sum, mod 65536, of the plaintext of all the algorithm-\nspecific octets (including MPI prefix and data).  With V3 keys, the\nchecksum is stored in the clear.  With V4 keys, the checksum is\nencrypted like the algorithm-specific data.  This value is used to\ncheck that the passphrase was correct.  However, this checksum is\ndeprecated; an implementation SHOULD NOT use it, but should rather\nuse the SHA-1 hash denoted with a usage octet of 254.  The reason for\nthis is that there are some attacks that involve undetectably\nmodifying the secret key.","ref":"OpenPGP.SecretKeyPacket.html"},{"type":"function","title":"OpenPGP.SecretKeyPacket.decode/1","doc":"Decode Secret Key Packet given input binary.\nReturn structured packet and remaining binary (empty binary).","ref":"OpenPGP.SecretKeyPacket.html#decode/1"},{"type":"function","title":"OpenPGP.SecretKeyPacket.decrypt/2","doc":"Decrypt Secret-Key Packet given decoded Secret-Key Packet and a\npassphrase.\nReturn Secret-Key Packet with `:secret_key_material` attr assigned.\nRaises an error if checksum does not match.","ref":"OpenPGP.SecretKeyPacket.html#decrypt/2"},{"type":"type","title":"OpenPGP.SecretKeyPacket.t/0","doc":"","ref":"OpenPGP.SecretKeyPacket.html#t:t/0"},{"type":"module","title":"OpenPGP.Radix64","doc":"Radix64 decoder, as per [RFC4880](https://www.ietf.org/rfc/rfc4880.txt)","ref":"OpenPGP.Radix64.html"},{"type":"function","title":"OpenPGP.Radix64.decode/1","doc":"","ref":"OpenPGP.Radix64.html#decode/1"},{"type":"module","title":"OpenPGP.Radix64.CRC24","doc":"CRC-24 implementation for Radix-64 checksum validation.\n\n---\n\n## [RFC4880](https://www.ietf.org/rfc/rfc4880.txt)\n\n#","ref":"OpenPGP.Radix64.CRC24.html"},{"type":"module","title":"6.  Radix-64 Conversions - OpenPGP.Radix64.CRC24","doc":"The checksum is a 24-bit Cyclic Redundancy Check (CRC) converted to\nfour characters of radix-64 encoding by the same MIME base64\ntransformation, preceded by an equal sign (=).  The CRC is computed\nby using the generator 0x864CFB and an initialization of 0xB704CE.\nThe accumulation is done on the data before it is converted to\nradix-64, rather than on the converted data.\n\n\n#","ref":"OpenPGP.Radix64.CRC24.html#module-6-radix-64-conversions"},{"type":"module","title":"6.1.  An Implementation of the CRC-24 in \"C\" - OpenPGP.Radix64.CRC24","doc":"```\n  #define CRC24_INIT 0xB704CEL\n  #define CRC24_POLY 0x1864CFBL\n\n  typedef long crc24;\n  crc24 crc_octets(unsigned char *octets, size_t len)\n  {\n      crc24 crc = CRC24_INIT;\n      int i;\n      while (len--) {\n          crc ^= (*octets++) << 16;\n          for (i = 0; i < 8; i++) {\n              crc <<= 1;\n              if (crc & 0x1000000)\n                  crc ^= CRC24_POLY;\n          }\n      }\n      return crc & 0xFFFFFFL;\n  }\n  ```","ref":"OpenPGP.Radix64.CRC24.html#module-6-1-an-implementation-of-the-crc-24-in-c"},{"type":"function","title":"OpenPGP.Radix64.CRC24.calc/1","doc":"Calculate CRC-24 of a given binary.\n\n#","ref":"OpenPGP.Radix64.CRC24.html#calc/1"},{"type":"function","title":"Example: - OpenPGP.Radix64.CRC24.calc/1","doc":"iex> OpenPGP.Radix64.CRC24.calc(\"Hello, world!!!\")\n    <<190, 125, 81>>","ref":"OpenPGP.Radix64.CRC24.html#calc/1-example"},{"type":"module","title":"OpenPGP.Radix64.Entry","doc":"Represents a block/entry in the PGP armored message.\n\n## [RFC4880](https://www.ietf.org/rfc/rfc4880.txt)\n\n6.2.  Forming ASCII Armor\n\n When OpenPGP encodes data into ASCII Armor, it puts specific headers\n around the Radix-64 encoded data, so OpenPGP can reconstruct the data\n later.  An OpenPGP implementation MAY use ASCII armor to protect raw\n binary data.  OpenPGP informs the user what kind of data is encoded\n in the ASCII armor through the use of the headers.\n\n Concatenating the following data creates ASCII Armor:\n\n   - An Armor Header Line, appropriate for the type of data\n\n   - Armor Headers\n\n   - A blank (zero-length, or containing only whitespace) line\n\n   - The ASCII-Armored data\n\n   - An Armor Checksum\n\n   - The Armor Tail, which depends on the Armor Header Line\n\n An Armor Header Line consists of the appropriate header line text\n surrounded by five (5) dashes ('-', 0x2D) on either side of the\n header line text.  The header line text is chosen based upon the type\n of data that is being encoded in Armor, and how it is being encoded.\n Header line texts include the following strings:\n\n BEGIN PGP MESSAGE\n     Used for signed, encrypted, or compressed files.\n\n BEGIN PGP PUBLIC KEY BLOCK\n     Used for armoring public keys.\n\n BEGIN PGP PRIVATE KEY BLOCK\n     Used for armoring private keys.\n\n BEGIN PGP MESSAGE, PART X/Y\n     Used for multi-part messages, where the armor is split amongst Y\n     parts, and this is the Xth part out of Y.\n\n BEGIN PGP MESSAGE, PART X\n     Used for multi-part messages, where this is the Xth part of an\n     unspecified number of parts.  Requires the MESSAGE-ID Armor\n     Header to be used.\n\n BEGIN PGP SIGNATURE\n     Used for detached signatures, OpenPGP/MIME signatures, and\n     cleartext signatures.  Note that PGP 2.x uses BEGIN PGP MESSAGE\n     for detached signatures.","ref":"OpenPGP.Radix64.Entry.html"},{"type":"type","title":"OpenPGP.Radix64.Entry.t/0","doc":"","ref":"OpenPGP.Radix64.Entry.html#t:t/0"},{"type":"extras","title":"OpenPGP","doc":"# OpenPGP\n\n[![Build Status](https://github.com/DivvyPayHQ/open_pgp/workflows/CI/badge.svg)](https://github.com/DivvyPayHQ/open_pgp/actions?query=workflow%3ACI)\n[![Hex pm](https://img.shields.io/hexpm/v/open_pgp.svg)](https://hex.pm/packages/open_pgp)\n[![Hex Docs](https://img.shields.io/badge/hex-docs-blue.svg)](https://hexdocs.pm/open_pgp/)\n[![License](https://img.shields.io/badge/License-MIT-blue.svg)](https://opensource.org/licenses/MIT)\n\nOpenPGP lib allows to inspect, decode and decrypt OpenPGP Message Format as per [RFC4880](https://www.ietf.org/rfc/rfc4880.html)","ref":"readme.html"},{"type":"extras","title":"Installation - OpenPGP","doc":"Add `:open_pgp` to the list of dependencies in `mix.exs`:\n\n```elixir\ndef deps() do\n  [\n    {:open_pgp, \"~> 0.5\"}\n  ]\nend\n```","ref":"readme.html#installation"},{"type":"extras","title":"OpenPGP Packet - OpenPGP","doc":"The `OpenPGP.Packet` is a generic packet type. It has an essential purpose: split OpenPGP message in packets and decode packet tags.\n\nAn OpenPGP message is constructed from a number of records that are traditionally called packets. A packet is a chunk of data that has a tag specifying its meaning. An OpenPGP message, keyring, certificate, and so forth consists of a number of packets. Some of those packets may contain other OpenPGP packets (for example, a compressed data packet, when uncompressed, contains OpenPGP packets). Each packet consists of a packet header, followed by the packet body. For more details refer to [Packet Syntax chapter in RFC4880](https://www.ietf.org/rfc/rfc4880.html#section-4)\n\nOnce OpenPGP message split into generic packets, the higher order tag-specific packet decoders can be applied on its' data. Example:\n\n```\n{packet, _rest} = OpenPGP.Packet.decode(\"...\")\n\n{compressed_data_packet, <<>>} =\n  packet |> OpenPGP.Util.concat_body() |> OpenPGP.CompressedDataPacket.decode()\n```\n\nMore details can be found in `OpenPGP.Packet` and `OpenPGP.Packet.Behaviour`","ref":"readme.html#openpgp-packet"},{"type":"extras","title":"Examples - OpenPGP","doc":"#","ref":"readme.html#examples"},{"type":"extras","title":"List and cast packets - OpenPGP","doc":"List packets in a message and then cast to specific packet types.\n\n```\niex> message = <<160, 24, 2, 120, 156, 243, 72, 205, 201, 201, 215, 81, 8, 207, 47, 202, 73, 81, 84, 84, 4, 0, 40, 213, 4, 172>>\n...>\niex> packets = OpenPGP.list_packets(message)\n[\n  %OpenPGP.Packet{\n    body: [\n      %OpenPGP.Packet.BodyChunk{\n        chunk_length: {:fixed, 24},\n        data: <<2, 120, 156, 243, 72, 205, 201, 201, 215, 81, 8, 207, 47, 202, 73, 81, 84, 84, 4, 0, 40, 213, 4, 172>>,\n        header_length: 1\n      }\n    ],\n    tag: %OpenPGP.Packet.PacketTag{\n      format: :old,\n      length_type: {0, \"one-octet\"},\n      tag: {8, \"Compressed Data Packet\"}\n    }\n  }\n]\niex> OpenPGP.cast_packets(packets)\n[\n  %OpenPGP.CompressedDataPacket{\n    algo: {2, \"ZLIB [RFC1950]\"},\n    data_deflated: <<120, 156, 243, 72, 205, 201, 201, 215, 81, 8, 207, 47, 202, 73, 81, 84, 84, 4, 0, 40, 213, 4, 172>>,\n    data_inflated: \"Hello, World!!!\"\n  }\n]\n```\n\n#","ref":"readme.html#list-and-cast-packets"},{"type":"extras","title":"Decode Generic OpenPGP packet - OpenPGP","doc":"In this example the packet tag specifies a Signature Packet with body length of 7 bytes. The remaining binary will be return as a second element in a two element tuple. More details in `OpenPGP.Packet.Behaviour`.\n\n```\niex> alias OpenPGP.Packet\niex> alias OpenPGP.Packet.PacketTag\niex> alias OpenPGP.Packet.BodyChunk\niex> Packet.decode(<<1::1, 0::1, 2::4, 0::2, 7::8, \"Hello, World!!!\">>)\n{\n  %Packet{\n    tag: %PacketTag{format: :old, length_type: {0, \"one-octet\"}, tag: {2, \"Signature Packet\"}},\n    body: [%BodyChunk{chunk_length: {:fixed, 7}, data: \"Hello, \", header_length: 1}]\n  },\n  \"World!!!\"\n}\n```\n\n#","ref":"readme.html#decode-generic-openpgp-packet"},{"type":"extras","title":"Load private key and decrypt PGP file - OpenPGP","doc":"This example assumes that the private key and encrypted message were exported in a raw binary format, which might not be the most common way of exporting PGP entries. If \"armored\" format ([Radix64](https://www.rfc-editor.org/rfc/rfc4880.html#section-6)) is used for exporting data (i.e., `gpg --armor --export ...`), you'll need to use `OpenPGP.Radix64.decode/1` first on file contents to get a list of entries and operate on its' `:data` attribute.\n\n```\nalias OpenPGP.Packet\nalias OpenPGP.Packet.PacketTag\nalias OpenPGP.CompressedDataPacket\nalias OpenPGP.IntegrityProtectedDataPacket\nalias OpenPGP.LiteralDataPacket\nalias OpenPGP.PublicKeyPacket\nalias OpenPGP.PublicKeyEncryptedSessionKeyPacket\nalias OpenPGP.SecretKeyPacket\n\n###################################\n#","ref":"readme.html#load-private-key-and-decrypt-pgp-file"},{"type":"extras","title":"Load encrypted message/file ### - OpenPGP","doc":"###################################\n\nencrypted_file = File.read!(\"test/fixtures/words.dict.gpg\")\n\n[\n  %PublicKeyEncryptedSessionKeyPacket{} = pkesk_packet,\n  %IntegrityProtectedDataPacket{} = ipdata_packet\n] = encrypted_file |> OpenPGP.list_packets() |> OpenPGP.cast_packets()\n\n%PublicKeyEncryptedSessionKeyPacket{public_key_id: public_key_id} = pkesk_packet\n\n#######################\n#","ref":"readme.html#load-encrypted-message-file"},{"type":"extras","title":"Load secret key ### - OpenPGP","doc":"#######################\n\nprivate_key_file = File.read!(\"test/fixtures/rsa2048-priv.pgp\")\npassphrase = \"passphrase\"\n\nkeyring =\n  [\n    %SecretKeyPacket{},\n    %Packet{tag: %PacketTag{tag: {13, \"User ID Packet\"}}},\n    %Packet{tag: %PacketTag{tag: {2, \"Signature Packet\"}}},\n    %SecretKeyPacket{},\n    %Packet{tag: %PacketTag{tag: {2, \"Signature Packet\"}}}\n  ] = private_key_file |> OpenPGP.list_packets() |> OpenPGP.cast_packets()\n\nsk_packet =\n  Enum.find_value(keyring, fn\n    %SecretKeyPacket{public_key: %PublicKeyPacket{id: ^public_key_id}} = packet -> packet\n    _ -> nil\n  end)\n\nsk_packet_decrypted = SecretKeyPacket.decrypt(sk_packet, passphrase)\n\n################################\n#","ref":"readme.html#load-secret-key"},{"type":"extras","title":"Decode encrypted message ### - OpenPGP","doc":"################################\n\npkesk_packet_decrypted = PublicKeyEncryptedSessionKeyPacket.decrypt(pkesk_packet, sk_packet_decrypted)\n\nipdata_packet_decrypted = IntegrityProtectedDataPacket.decrypt(ipdata_packet, pkesk_packet_decrypted)\n\n%IntegrityProtectedDataPacket{\n  version: 1,\n  ciphertext: \"\" <> _,\n  plaintext: plaintext\n} = ipdata_packet_decrypted\n\n[\n  %CompressedDataPacket{\n    algo: {2, \"ZLIB [RFC1950]\"},\n    data_deflated: <<_::bitstring>>,\n    data_inflated: data_inflated\n  }\n] = plaintext |> OpenPGP.list_packets() |> OpenPGP.cast_packets()\n\n[\n  %LiteralDataPacket{\n    format: {<<0x62>>, :binary},\n    file_name: \"words.dict\",\n    created_at: ~U[2024-01-04 00:27:32Z],\n    data: data\n  }\n] = data_inflated |> OpenPGP.list_packets() |> OpenPGP.cast_packets()\n\nIO.puts(data)\n```\n\n#","ref":"readme.html#decode-encrypted-message"},{"type":"extras","title":"CompressedDataPacket - OpenPGP","doc":"The `OpenPGP.CompressedDataPacket` will inflate data implicitly when decoded (also, data inflated implicitly when `OpenPGP.cast_packets/1` used).\n\n```\niex> alias OpenPGP.CompressedDataPacket\niex> deflated = <<120, 156, 243, 72, 205, 201, 201, 215, 81, 8, 207, 47, 202, 73, 81, 84, 84, 4, 0, 40, 213, 4, 172>>\niex> CompressedDataPacket.decode(<<2, deflated::binary>>)\n{\n  %CompressedDataPacket{\n    algo: {2, \"ZLIB [RFC1950]\"},\n    data_deflated: deflated,\n    data_inflated: \"Hello, World!!!\"},\n  <<>>\n}\n```\n\n#","ref":"readme.html#compresseddatapacket"},{"type":"extras","title":"IntegrityProtectedDataPacket - OpenPGP","doc":"The `OpenPGP.IntegrityProtectedDataPacket` does not decrypt its' data implicitly. The `OpenPGP.IntegrityProtectedDataPacket.decrypt/2` should be used to get plaintext. Please note that some packets have packet speicifc functions, such as `OpenPGP.IntegrityProtectedDataPacket.decrypt/2`.\n\n```\niex> alias OpenPGP.IntegrityProtectedDataPacket\niex> alias OpenPGP.PublicKeyEncryptedSessionKeyPacket\n...>\niex> key = <<38, 165, 130, 172, 168, 51, 184, 238, 96, 204, 88,\n...>   134, 93, 25, 162, 22, 83, 211, 140, 176, 115, 113, 37, 201,\n...>   171, 249, 115, 64, 94, 59, 35, 60>>\n...>\niex> iv = <<0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0>>\niex> < > = :crypto.strong_rand_bytes(16)\niex> plaintext = < >\n...>\niex> ciphertext =\n...>   :crypto.crypto_one_time(\n...>    :aes_256_cfb128,\n...>    key,\n...>    iv,\n...>    plaintext,\n...>    true)\n...>\niex> payload = <<1::8, ciphertext::binary>>\niex> {packet_decoded, <<>>} = IntegrityProtectedDataPacket.decode(payload)\n{\n  %IntegrityProtectedDataPacket{\n    ciphertext: ciphertext,\n    plaintext: nil,\n    version: 1\n  },\n  <<>>\n}\niex> pkesk = %PublicKeyEncryptedSessionKeyPacket{\n...>   version: 3,\n...>   session_key_algo: {9, \"AES with 256-bit key\"},\n...>   session_key_material: {key}\n...> }\n...>\niex> IntegrityProtectedDataPacket.decrypt(packet_decoded, pkesk)\n%IntegrityProtectedDataPacket{\n  version: 1,\n  plaintext: \"Hello\",\n  ciphertext: ciphertext\n}\n```\n\n#","ref":"readme.html#integrityprotecteddatapacket"},{"type":"extras","title":"Radix64 / Armored format - OpenPGP","doc":"```\npayload = \"\"\"\n-----BEGIN PGP MESSAGE-----\nVersion: OpenPrivacy 0.99\n\nyDgBO22WxBHv7O8X7O/jygAEzol56iUKiXmV+XmpCtmpqQUKiQrFqclFqUDBovzS\nvBSFjNSiVHsuAA==\n=njUN\n-----END PGP MESSAGE-----\n\"\"\"\n\n[\n  %Radix64.Entry{\n    crc: <<158, 53, 13>>,\n    data: <<200, 56, 1, 59, _::binary>> = data,\n    meta: [{\"Version\", \"OpenPrivacy 0.99\"}],\n    name: \"PGP MESSAGE\"\n  }\n] = OpenPGP.Radix64.decode(payload)\n```","ref":"readme.html#radix64-armored-format"},{"type":"extras","title":"Notes - OpenPGP","doc":"As of **v0.5.x**:\n\n1. Any valid OpenPGP message can be decoded via generic `OpenPGP.Packet` decoder. This abstraction layer provide Packet Tags and Body Chunks for packet envelope level evaluation.\n1. Some Packet Tag specific decoders implemented with limited feature support:\n   1. `OpenPGP.LiteralDataPacket`\n   1. `OpenPGP.PublicKeyEncryptedSessionKeyPacket`\n   1. `OpenPGP.PublicKeyPacket` - support only V4 packets\n   1. `OpenPGP.SecretKeyPacket` - support only V4 packets; Iterated and Salted String-to-Key (S2K) specifier (ID: 3); S2K usage convention octet of 254 only; S2K hashing algo SHA1; AES128 symmetric encryption of secret key material\n   1. `OpenPGP.CompressedDataPacket` - support only ZLIB- and ZIP-style blocks\n   1. `OpenPGP.IntegrityProtectedDataPacket` - support Session Key algo 9 (AES with 256-bit key) in CFB mode; Modification Detection Code system is not supported\n\nAt a high level `OpenPGP.list_packets/1` and `OpenPGP.cast_packets/1` serve as an entrypoint to OpenPGP Message decoding and extracting generic data.\n\nPacket specific decoders implement `OpenPGP.Packet.Behaviour`, which exposes `.decode/1` interface (including genric `OpenPGP.Packet`). Additionaly some of the packet specific decoders may provide interface for further packet processing, such as `OpenPGP.SecretKeyPacket.decrypt/2`.\n\nUsage example of a comon use case can be found in `test/open_pgp/open_pgp_test.exs` in the test **\"full integration: load private key and decrypt encrypted file\"**","ref":"readme.html#notes"},{"type":"extras","title":"Refs, Snippets, Misc - OpenPGP","doc":"```console\n# GPG commands\n~$ gpg --list-keys\n~$ gpg --list-secret-keys\n~$ gpg --export-secret-key --armor john.doe@example.com > ./private.pgp\n~$ gpg --list-packets --verbose example.txt.pgp\n~$ gpg --encrypt --recipient F89B64F782254B03624FCF5C052E8381B5C335DA /usr/share/dict/words\n~$ gpg --batch --passphrase \"passphrase\" --quick-generate-key \"John Doe (RSA2048)  \" rsa2048 default never\n~$ gpg --edit-key F89B64F782254B03624FCF5C052E8381B5C335DA\n\n# Handy tools\n~$ hexdump -vx ./words.pgp\n~$ xxd -b ./words.pgp\n~$ xxd -g 1\n```","ref":"readme.html#refs-snippets-misc"},{"type":"extras","title":"License","doc":"# License\n\nMIT License\n\nCopyright (c) 2024 BILL Operations, LLC.\n\nPermission is hereby granted, free of charge, to any person obtaining a copy\nof this software and associated documentation files (the \"Software\"), to deal\nin the Software without restriction, including without limitation the rights\nto use, copy, modify, merge, publish, distribute, sublicense, and/or sell\ncopies of the Software, and to permit persons to whom the Software is\nfurnished to do so, subject to the following conditions:\n\nThe above copyright notice and this permission notice shall be included in all\ncopies or substantial portions of the Software.\n\nTHE SOFTWARE IS PROVIDED \"AS IS\", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR\nIMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,\nFITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE\nAUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER\nLIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,\nOUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE\nSOFTWARE.","ref":"license.html"},{"type":"extras","title":"Code of Conduct","doc":"# Code of Conduct","ref":"code_of_conduct.html"},{"type":"extras","title":"Our Pledge - Code of Conduct","doc":"We as members, contributors, and leaders pledge to make participation in our community a harassment-free experience for everyone, regardless of age, body size, visible or invisible disability, ethnicity, sex characteristics, gender identity and expression, level of experience, education, socio-economic status, nationality, personal appearance, race, religion, or sexual identity and orientation.\n\nWe pledge to act and interact in ways that contribute to an open, welcoming, diverse, inclusive, and healthy community.","ref":"code_of_conduct.html#our-pledge"},{"type":"extras","title":"Our Standards - Code of Conduct","doc":"Examples of behavior that contributes to a positive environment for our community include:\n\n- Demonstrating empathy and kindness toward other people\n- Being respectful of differing opinions, viewpoints, and experiences\n- Giving and gracefully accepting constructive feedback\n- Accepting responsibility and apologizing to those affected by our mistakes, and learning from the experience\n- Focusing on what is best not just for us as individuals, but for the overall community\n\nExamples of unacceptable behavior include:\n\n- The use of sexualized language or imagery, and sexual attention or\n  advances of any kind\n- Trolling, insulting or derogatory comments, and personal or political attacks\n- Public or private harassment\n- Publishing others' private information, such as a physical or email\n  address, without their explicit permission\n- Other conduct which could reasonably be considered inappropriate in a\n  professional setting","ref":"code_of_conduct.html#our-standards"},{"type":"extras","title":"Enforcement Responsibilities - Code of Conduct","doc":"Community leaders are responsible for clarifying and enforcing our standards of acceptable behavior and will take appropriate and fair corrective action in response to any behavior that they deem inappropriate, threatening, offensive, or harmful.\n\nCommunity leaders have the right and responsibility to remove, edit, or reject comments, commits, code, wiki edits, issues, and other contributions that are not aligned to this Code of Conduct, and will communicate reasons for moderation decisions when appropriate.","ref":"code_of_conduct.html#enforcement-responsibilities"},{"type":"extras","title":"Scope - Code of Conduct","doc":"This Code of Conduct applies within all community spaces, and also applies when an individual is officially representing the community in public spaces. Examples of representing our community include using an official e-mail address, posting via an official social media account, or acting as an appointed representative at an online or offline event.","ref":"code_of_conduct.html#scope"},{"type":"extras","title":"Enforcement - Code of Conduct","doc":"Instances of abusive, harassing, or otherwise unacceptable behavior may be reported to the community leaders responsible for enforcement at [INSERT CONTACT METHOD]. All complaints will be reviewed and investigated promptly and fairly.\n\nAll community leaders are obligated to respect the privacy and security of the reporter of any incident.","ref":"code_of_conduct.html#enforcement"},{"type":"extras","title":"Enforcement Guidelines - Code of Conduct","doc":"Community leaders will follow these Community Impact Guidelines in determining the consequences for any action they deem in violation of this Code of Conduct:\n\n#","ref":"code_of_conduct.html#enforcement-guidelines"},{"type":"extras","title":"1. Correction - Code of Conduct","doc":"**Community Impact**: Use of inappropriate language or other behavior deemed unprofessional or unwelcome in the community.\n\n**Consequence**: A private, written warning from community leaders, providing clarity around the nature of the violation and an explanation of why the behavior was inappropriate. A public apology may be requested.\n\n#","ref":"code_of_conduct.html#1-correction"},{"type":"extras","title":"2. Warning - Code of Conduct","doc":"**Community Impact**: A violation through a single incident or series of actions.\n\n**Consequence**: A warning with consequences for continued behavior. No interaction with the people involved, including unsolicited interaction with those enforcing the Code of Conduct, for a specified period of time. This includes avoiding interactions in community spaces as well as external channels like social media. Violating these terms may lead to a temporary or permanent ban.\n\n#","ref":"code_of_conduct.html#2-warning"},{"type":"extras","title":"3. Temporary Ban - Code of Conduct","doc":"**Community Impact**: A serious violation of community standards, including sustained inappropriate behavior.\n\n**Consequence**: A temporary ban from any sort of interaction or public communication with the community for a specified period of time. No public or private interaction with the people involved, including unsolicited interaction with those enforcing the Code of Conduct, is allowed during this period. Violating these terms may lead to a permanent ban.\n\n#","ref":"code_of_conduct.html#3-temporary-ban"},{"type":"extras","title":"4. Permanent Ban - Code of Conduct","doc":"**Community Impact**: Demonstrating a pattern of violation of community standards, including sustained inappropriate behavior, harassment of an individual, or aggression toward or disparagement of classes of individuals.\n\n**Consequence**: A permanent ban from any sort of public interaction within the community.","ref":"code_of_conduct.html#4-permanent-ban"},{"type":"extras","title":"Attribution - Code of Conduct","doc":"This Code of Conduct is adapted from the [Contributor Covenant][homepage], version 2.0,\navailable at https://www.contributor-covenant.org/version/2/0/code_of_conduct.html.\n\nCommunity Impact Guidelines were inspired by [Mozilla's code of conduct enforcement ladder](https://github.com/mozilla/diversity).\n\n[homepage]: https://www.contributor-covenant.org\n\nFor answers to common questions about this code of conduct, see the FAQ at\nhttps://www.contributor-covenant.org/faq. Translations are available at https://www.contributor-covenant.org/translations.","ref":"code_of_conduct.html#attribution"},{"type":"extras","title":"Changelog","doc":"# Changelog","ref":"changelog.html"}],"content_type":"text/markdown"}